//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Mas Awoo"
#property link      "mailto:mas_awoo@yahoo.com"
//------------------------------------------------

#define MAGICMA  20050610
extern double MinLots            = 0.1;
extern double MaxLotSize         = 111;
extern double StopLoss           = 299;
extern double TakeProfit1        = 75;
extern double TakeProfit2        = 100;
extern double TakeProfit3        = 125;
extern double TakeProfit4        = 150;
extern double TakeProfit5        = 1111;
extern double TrailingStop       = 99;
extern double MaximumRisk        = 0.02;
extern double IncreaseFactor     = 2;
extern double MA_PERIODE         = 200;
extern double ATR_PERIODE        = 100;

//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=MinLots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0, profit=0;                  // number of losses orders without a break
//---- select lot size
//---- calcuulate number of losses orders without a break
   if(IncreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) profit++;
         if(OrderProfit()<0) losses++;lot++; 
        }
      if(losses>1) lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk*2/1000.0,1);
      if(profit>1) lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
     }
//---- return lot size
   if(lot<MinLots) lot=MinLots;
   if(lot>MaxLotSize) lot=MaxLotSize;
   return(lot);
  }
  
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double AC1, AC2, AC11, AC22, AO1, AO2,St, Sts, Atr, MA;
   int    res;
//---- go trading only for first tiks of new bar
   if (Volume[0]>1)return;
//---- get Moving Average 
   St=iStochastic(NULL,0,18,5,5,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,5,3,3,MODE_SMA,0,MODE_SIGNAL,0);
   AC1=iCustom(NULL, 0, "AC1",1,0);
   AC2=iCustom(NULL, 0, "AC2",2,0);
   AC11=iCustom(NULL, 0, "AC1",1,1);
   AC22=iCustom(NULL, 0, "AC2",2,1);
   Atr=iForce(NULL, 0, ATR_PERIODE,MODE_SMA,PRICE_CLOSE,1);
   MA=iMA(NULL,0,MA_PERIODE,0,MODE_SMA,0,0);
   

//---- Buy conditions
   if(AC1>0 && AC11<0 && St>80 && St>Sts && Atr>0 && iOpen(NULL,0,0)>MA)
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit1*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit2*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit3*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit4*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,Ask-StopLoss*Point,Ask+TakeProfit5*Point,"Mas Awoo",MAGICMA,0,Red);
      return;
     }

  //----
//---- Sell conditions
   if(AC2<0 && AC22>0 && St<20 && St<Sts && Atr<0 && iOpen(NULL,0,0)<MA)
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit1*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit2*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit3*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit4*Point,"Mas Awoo",MAGICMA,0,Red);
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,Bid+StopLoss*Point,Bid-TakeProfit5*Point,"Mas Awoo",MAGICMA,0,Red);
      return;
     }

  //----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double AC1, AC2, AC11, AC22, AO1, AO2,St, Sts, Atr, MA;
   int    res;
//---- go trading only for first tiks of new bar
//---- get Moving Average 
   St=iStochastic(NULL,0,18,5,5,MODE_SMA,0,MODE_MAIN,0);
   Sts=iStochastic(NULL,0,5,3,3,MODE_SMA,0,MODE_SIGNAL,0);
   AC1=iCustom(NULL, 0, "AC1",1,0);
   AC2=iCustom(NULL, 0, "AC2",2,0);
   AC11=iCustom(NULL, 0, "AC1",1,1);
   AC22=iCustom(NULL, 0, "AC2",2,1);
   Atr=iForce(NULL, 0, ATR_PERIODE,MODE_SMA,PRICE_CLOSE,1);
   MA=iMA(NULL,0,MA_PERIODE,0,MODE_SMA,0,0);
   

//---- get Moving Average 
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(AC2<0 && AC22>0 && St<20 && St<Sts && Atr<0&& iOpen(NULL,0,0)<MA) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(AC1>0 && AC11<0 && St>80 && St>Sts && Atr>0 && iOpen(NULL,0,0)>MA) OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }

  }
//+------------------------------------------------------------------+
//| Check For TrailingStop                                           |
//+------------------------------------------------------------------+
void TrailingStop ()
{
double total, cnt;
   total=OrdersTotal();
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL && 
         OrderSymbol()==Symbol())   
        {
         if(OrderType()==OP_BUY)  
           {
            if(TrailingStop>0)  
              {                 
               if(Bid-OrderOpenPrice()>Point*TrailingStop)
                 {
                  if(OrderStopLoss()<Bid-Point*TrailingStop)
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Red);
                     return(0);
                    }
                 }
              }
           }
         else //
           {
            if(TrailingStop>0)  
              {                 
               if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
                 {
                  if(OrderStopLoss()==0.0 || 
                     OrderStopLoss()>(Ask+Point*TrailingStop))
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red);
                     return(0);
                    }
                 }
              }
           }
        }
     }
     return(0);
     }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) 
   CheckForOpen();
   else CheckForClose();
   TrailingStop ();
//----
  }
//+------------------------------------------------------------------+